@@ -361,6 +361,7 @@ int (*scheme_bucket_table_index)(Scheme_Bucket_Table *hash, mzlonglong pos, Sche
 Scheme_Object *(*scheme_bucket_table_next)(Scheme_Bucket_Table *hash, mzlonglong start);
 Scheme_Hash_Table *(*scheme_make_hash_table)(int type);
 Scheme_Hash_Table *(*scheme_make_hash_table_equal)();
+Scheme_Hash_Table *(*scheme_make_hash_table_equal_always)();
 Scheme_Hash_Table *(*scheme_make_hash_table_eqv)();
 void (*scheme_hash_set)(Scheme_Hash_Table *table, Scheme_Object *key, Scheme_Object *val);
 Scheme_Object *(*scheme_hash_get)(Scheme_Hash_Table *table, Scheme_Object *key);
@@ -369,6 +370,7 @@ void (*scheme_hash_set_atomic)(Scheme_Hash_Table *table, Scheme_Object *key, Sch
 Scheme_Object *(*scheme_hash_get_atomic)(Scheme_Hash_Table *table, Scheme_Object *key);
 int (*scheme_hash_table_equal)(Scheme_Hash_Table *t1, Scheme_Hash_Table *t2);
 int (*scheme_is_hash_table_equal)(Scheme_Object *o);
+int (*scheme_is_hash_table_equal_always)(Scheme_Object *o);
 int (*scheme_is_hash_table_eqv)(Scheme_Object *o);
 Scheme_Hash_Table *(*scheme_clone_hash_table)(Scheme_Hash_Table *ht);
 void (*scheme_clear_hash_table)(Scheme_Hash_Table *ht);
@@ -382,6 +384,7 @@ mzlonglong (*scheme_hash_tree_next)(Scheme_Hash_Tree *tree, mzlonglong pos);
 int (*scheme_hash_tree_index)(Scheme_Hash_Tree *tree, mzlonglong pos, Scheme_Object **_key, Scheme_Object **_val);
 int (*scheme_hash_tree_equal)(Scheme_Hash_Tree *t1, Scheme_Hash_Tree *t2);
 int (*scheme_is_hash_tree_equal)(Scheme_Object *o);
+int (*scheme_is_hash_tree_equal_always)(Scheme_Object *o);
 int (*scheme_is_hash_tree_eqv)(Scheme_Object *o);
 /*========================================================================*/
 /*                   basic Scheme value constructors                      */
@@ -895,6 +898,7 @@ int (*scheme_is_subinspector)(Scheme_Object *i, Scheme_Object *sup);
 int (*scheme_eq)(Scheme_Object *obj1, Scheme_Object *obj2);
 int (*scheme_eqv)(Scheme_Object *obj1, Scheme_Object *obj2);
 int (*scheme_equal)(Scheme_Object *obj1, Scheme_Object *obj2);
+int (*scheme_equal_always)(Scheme_Object *obj1, Scheme_Object *obj2);
 int (*scheme_chaperone_of)(Scheme_Object *obj1, Scheme_Object *obj2);
 int (*scheme_impersonator_of)(Scheme_Object *obj1, Scheme_Object *obj2);
 #ifdef MZ_PRECISE_GC
@@ -904,6 +908,8 @@ intptr_t (*scheme_equal_hash_key)(Scheme_Object *o);
 intptr_t (*scheme_equal_hash_key2)(Scheme_Object *o);
 intptr_t (*scheme_recur_equal_hash_key)(Scheme_Object *o, void *cycle_data);
 intptr_t (*scheme_recur_equal_hash_key2)(Scheme_Object *o, void *cycle_data);
+intptr_t (*scheme_equal_always_hash_key)(Scheme_Object *o);
+intptr_t (*scheme_equal_always_hash_key2)(Scheme_Object *o);
 intptr_t (*scheme_eqv_hash_key)(Scheme_Object *o);
 intptr_t (*scheme_eqv_hash_key2)(Scheme_Object *o);
 void (*scheme_set_type_equality)(Scheme_Type type, 
